;SSSD audio defines

.include 'tao'

.if !?def(__SSSD_INC)
__SSSD_INC = 1

; dummy typedefs for internal structures
; NOTE this can be replaced by "forward" in intent 1.3.3+
record ga_sample
endrecord

record ga_playing
endrecord

; NOTE:  APPLICATION SHOULD NEVER MODIFY THIS STRUCT DIRECTLY
record sssdaudio
    int channels
    int freq
    int bits
    int max_voices
    int max_samples
    int byps
    
    int audioBufferSize
    int numAudioBuffers
    
    int flags;
    int lastERR
    
    int paused
    int background_stream_bytes
    int background_flags
    int audio_fd
    void [background_stream]
    
    int num_samples
    int num_playing
    int playing_count

    ga_sample [samples]
    ga_playing [playing]
    
    void [playBuffer]
    
    int flagsOS
    
    void [buffer_mutex]
    void [audio_thread]

	void [platformData]		; Platform specific struct pointer
endrecord

;system flags
SSSDSYS_CONSTRATE = 1

; flagsOS
dbitstart
	dbit OS_LINUX,OS_LINUX_B
	dbit OS_WIN32,OS_WIN32_B
	dbit OS_WINCE,OS_WINCE_B
	dbit OS_SYMB6,OS_SYMB6_B
	dbit OS_AMIGA,OS_AMIGA_B
	dbit OS_AMIDE,OS_AMIDE_B

OS_ADEHO = $100
OS_ADEHO_B = 8

; Channels

LOAD_DEFAULT = 1
LOAD_STEREO	 = 1
LOAD_LEFT	 = 2
LOAD_RIGHT	 = 4
LOAD_DONT_RESAMPLE = 8

; samplePlay options 

DEFAULT_DEVICE = NULL
DEFAULT_NUMBUFFERS = 0
LOOP_FOREVER = 0
SAMPLE_DEFAULTPOS = -1
PRIQ_NONE = 100

; Background flags 
BKG_FORMAT_BESTGUESS = $00
BKG_FORMAT_MP3       = $01
BKG_FORMAT_OGG       = $02
BKG_FORMAT_MOD       = $03
BKG_FORMAT_USER      = $FF

BKG_PLAY_LOOP        = $100

; getAudioInfoData
structure
	byte AI_CHANNELS
	byte AI_FREQ
	byte AI_BITS
	byte AI_MAXVOICES
	byte AI_PLAYINGVOICES
	byte AI_MAXSAMPLES
	byte AI_SAMPLES
	byte AI_PAUSED
	byte AI_BGSTREAM
	byte AI_BUFFERSIZE
	byte AI_NUMINFOITEMS

; ctrlSample() ID Tags

CTRL_S_END    	= $00
CTRL_S_VOLUME 	= $10
CTRL_S_PAN    	= $11
CTRL_S_PLAYSTART = $12
CTRL_S_PLAYEND   = $13
CTRL_S_LOOPSTART = $14
CTRL_S_LOOPEND   = $15
CTRL_S_PRIORITY  = $16
CTRL_S_LOOPCOUNT = $17
CTRL_S_ARRAY     = $FF

; ctrlDevice() ID Tags

CTRL_D_END		= $00
CTRL_D_VOLUME	= $10
CTRL_D_PAN		= $11
CTRL_D_ARRAY    = $FF

; getSystemInfo bytes and defines 

;getSystemInfoBytes
structure
   byte SYS_FREQ
   byte SYS_BITS
   byte SYS_CHAN
   byte SYS_MISC
   byte SYS_CPUA
   byte SYS_CPUL
   byte SYS_CPUR
   byte SYS_GHZ
   byte SYS_MHZ
   byte SYS_OS
   byte SYS_OSV1
   byte SYS_OSV2
   byte SYS_OSVB
   byte SYS_ENDI
   byte SYS_NUMVARS

dbitstart
	dbit SYS_FREQ_ARBITRARY, SYS_FREQ_ARBITRARY_B
	dbit SYS_FREQ_8000,		 SYS_FREQ_8000_B 
	dbit SYS_FREQ_11025,	 SYS_FREQ_11025_B
	dbit SYS_FREQ_22050,	 SYS_FREQ_22050_B
	dbit SYS_FREQ_44100,	 SYS_FREQ_44100_B
	dbit SYS_FREQ_48000,	 SYS_FREQ_48000_B
	dbit SYS_FREQ_96000,	 SYS_FREQ_96000_B
   
dbitstart
	dbit SYS_BITS_8,	 SYS_BITS_8_B
	dbit SYS_BITS_16,    SYS_BITS_16_B   
	dbit SYS_BITS_24,    SYS_BITS_24_B   
	dbit SYS_BITS_32,    SYS_BITS_32_B   
	dbit SYS_BITS_FLOAT, SYS_BITS_FLOAT_B

dbitstart
	dbit SYS_MISC_DSP, 	 SYS_MISC_DSP_B 
	dbit SYS_MISC_ARAM,	 SYS_MISC_ARAM_B


; ERROR CODES 

ERR_NOERROR 		= 0

ERR_NOT_INITIALIZED = -32001
ERR_DEVICE_OPEN     = -32002
ERR_CTRL_ALLOC      = -32003
ERR_NOT_SUPPORTED   = -32004 ; By 1.0 user should never get this error 
ERR_MEMORY_ALLOC    = -32005

ERR_PLAYBUFFER_ALLOC = -32007
ERR_SYSINFO         = -32009

ERR_MUTEX_INIT      = -32010
ERR_THREAD_START    = -32011
ERR_THREAD_STOP     = -32012

ERR_SAMPLES_FULL  	= -32020
ERR_SAMPLES_ALLOC 	= -32021
ERR_SAMPLES_FILE  	= -32022
ERR_SAMPLES_ATTRIB	= -32023
ERR_SAMPLES_RANGE 	= -32024

ERR_BKG_START 		= -32030
ERR_BKG_FORMAT 		= -32031
ERR_BKG_NOTPLAYING	= -32032

ERR_VOICES_ALLOC    =  -32040
ERR_VOICES_FULL     =  -32041

ERR_PLAYING_ID      =  -32045
ERR_PLAYING_LOOP    =  -32046

ERR_DEVICE_UNSUPPORTED	= -32050
ERR_DEVICE_UNKNOWN      = -32051
ERR_DEVICE_SUSPEND      = -32055
ERR_DEVICE_RESUME       = -32056

ERR_HARDWARE_PARMS	= -32070
ERR_HARDWARE_BITS	= -32071
ERR_HARDWARE_CHAN	= -32072
ERR_HARDWARE_FREQ	= -32073
ERR_HARDWARE_FRAG	= -32074

; internal macros to parse argument set	and add extra bits

.macro sssd_copyInternal
	.check %N = 8           ; copy sample: wrong number of args
	; separate last input from output
	__input := %7{1,':'}
	__output := %7{':',')'}
	; make the actual call
	qcall ami/sys/audio/sssd/sssd_copySampleStereo,(%1,%2,%3,%4,%5,%6,__input|%8 : __output)
.endm

.macro sssd_playInternal
	.check %N = 5           ; playSample: wrong number of args
	; separate last input from output
	__input := %5{1,':'}
	__output := %5{':',')'}
	; make the actual call
	qcall ami/sys/audio/sssd/sssd_playSamplePP,(%1,%2,0,%3,%4,__input,SAMPLE_DEFAULTPOS, SAMPLE_DEFAULTPOS, SAMPLE_DEFAULTPOS, SAMPLE_DEFAULTPOS, PRIQ_NONE : __output)
.endm

; Macros 

.define sssd_isError(a)		(a < -32000)
.define sssd_audioValid(ga) ([ga.audio_fd] > -1)
.define sssd_audioPaused(ga) ([ga.paused] != 0)
.define audioValid(ga)		([ga.audio_fd] > -1)
.define audioPaused(ga)		([ga.paused] != 0

 
.macro sssd_copySample
	.check %N = 1		; sssd_copySample (sssdaudio [ga], void [data], int channels, int freq, int bits, int samples, int flags : int)
	sssd_copyInternal %1{'(',')'},LOAD_STEREO	; remove parentheses from input	to allow parsing of separate args
.endm

.macro sssd_copySampleLeft
	.check %N = 1		; sssd_copySampleLeft (sssdaudio [ga], void [data], int channels, int freq, int bits, int samples, int flags : int)
	sssd_copyInternal %1{'(',')'},LOAD_LEFT
.endm

.macro sssd_copySampleRight
	.check %N = 1		; sssd_copySampleRight (sssdaudio [ga], void [data], int channels, int freq, int bits, int samples, int flags : int)
	sssd_copyInternal %1{'(',')'},LOAD_RIGHT
.endm

.macro sssd_playSample
	.check %N = 1		; sssd_playSample (a,b,c,d,e : int)
	sssd_playInternal %1{'(',')'}
.endm

.endif	; __SSSD_INC
