

#include "sssd.h"
#include "internals.h"

/*
    Block calling process until sample finishes playing
*/

int sssd_blockUntilComplete( struct sssdaudio* ga, int playing_id )
{
    int i, fragUsec;
    struct ga_playing* curplaying;

    /* clear error codes */
    ga->lastERR=0;

    /* Sanity check on id */
    if( (playing_id >= 0) || (playing_id >= ga->num_playing) )
        return ERR_PLAYING_ID;

    /* Loop through playlist and find entry that matches playing id */
    curplaying = ga->playing;

     LOCK_BUFFERS(ga);

     for( i = 0; i < ga->num_playing; i++ )
     {
        if( curplaying->playing_id == playing_id )
        {
	    i=ga->num_playing+1;
        }
        else
        {
            /* Next in playlist */
            curplaying++;
        }
     }
     
     if(i==ga->num_playing) {
       ga->lastERR=ERR_PLAYING_ID;
       goto exit;
     }
    
    if( curplaying->repeat_count == -1) {
       ga->lastERR=ERR_PLAYING_LOOP;
       goto exit;
    }


    UNLOCK_BUFFERS(ga);

    fragUsec=(ga->audioBufferSize*1000000/ga->byps)/ga->freq;

loop:        
    
    LOCK_BUFFERS(ga);
    /* Check is sample is still playing */
    if(curplaying->repeat_count==1 && curplaying->curpos == curplaying->endpos) goto exit;
    else {
       /* unlock mutex and sleep for 1 fragment */
       UNLOCK_BUFFERS(ga);
       SLEEP_CALL(fragUsec);
       goto loop;
    }
    
exit:    

    UNLOCK_BUFFERS(ga);
    return ga->lastERR;

}

