
/* SSSD audio defines */

#ifndef __SSSD_H
#define __SSSD_H

#include <stdio.h>

/* System specific defines */

#ifdef __LINUX__
   /* thread and mutex stuff */
   #include <pthread.h>
   #define MUTEX_TYPE pthread_mutex_t
   #define THREAD_TYPE pthread_t
   
   /* No proto modifier needed for Linux */
   #define _WINDOWS_DLL_PROTO

   
#endif /* Linux */



#ifdef __ELATE__
   #include <elate/kn.h>
   #ifdef __ELATE_HOSTED__
      #define _WINDOWS_DLL_PROTO extern
   #else
      #define _WINDOWS_DLL_PROTO
   #endif

   #define MUTEX_TYPE ELATE_MUTEX
   #define THREAD_TYPE void*

#endif /* Elate */



#ifdef _WIN32_WCE
   #include <windows.h>
   #include <winbase.h>
   #include <mmsystem.h>

   #ifdef BUILD_DLL
      #define _WINDOWS_DLL_PROTO __declspec(dllexport)
   #else
      #define _WINDOWS_DLL_PROTO __declspec(dllimport)
   #endif

   #define MUTEX_TYPE HANDLE
   #define THREAD_TYPE HANDLE


#endif /* WIN32_WCE */


#ifdef __SYMBIAN32__
    #define MUTEX_TYPE void *
    #define THREAD_TYPE void *

   /* No proto modifier needed for Symbian */
   #define _WINDOWS_DLL_PROTO
   #define __NOPRINTING__

#endif /* SYMBIAN32 */

#define DEFAULT_DEVICE NULL

   /* Other possible platforms */
   #define ESD_DEVICE_NAME ":esd"
   #define ARTS_DEVICE_NAME ":aRts"

#define DEFAULT_NUMBUFFERS 0


/* NOTE:  APPLICATION SHOULD NEVER MODIFY THIS STRUCT DIRECTLY */
struct sssdaudio
{
    int channels;
    int freq;
    int bits;
    int max_voices;
    int max_samples;
    int byps;
       
    int audioBufferSize;
    int numAudioBuffers;
    
    int flags;
    int lastERR;
    
    int paused;
    int background_stream_bytes;
    int background_flags;
    int audio_fd;
    FILE* background_stream;
    
    int num_samples;
    int num_playing;
    int playing_count;

    struct ga_sample *samples;    
    struct ga_playing *playing;
    
    void *playBuffer;
    
    int flagsOS;
    
    int threadSignal;
    
    MUTEX_TYPE buffer_mutex;
    THREAD_TYPE audio_thread;

    void *platformData;   /* Platform specific struct pointer */

};

/* System flags */
#define SSSDSYS_CONSTRATE 0x01

/* flagsOS */
#define OS_LINUX 0x01
#define OS_WIN32 0x02
#define OS_WINCE 0x04
#define OS_SYMB6 0x08
#define OS_AMIGA 0x10
#define OS_AMIDE 0x20

#define OS_ADEHO 0x100


/* Channels */

#define COPY_DEFAULT    0x01
#define COPY_STEREO	0x01
#define COPY_LEFT	0x02
#define COPY_RIGHT	0x04
#define COPY_DONT_RESAMPLE 0x08

#define RESAMPLE_NONE   0x08
#define RESAMPLE_FAST   0x10
#define RESAMPLE_QUALITY 0x20   
#define RESAMPLE_BEST   0x60 /* both QUALITY and InterpFilter */

#define RESAMPLE_MASK 0x78

/* samplePlay options */

#define LOOP_FOREVER 0
#define SAMPLE_DEFAULTPOS -1
#define PRIQ_NONE 100


/* Background flags */
#define BKG_FORMAT_BESTGUESS 0x00
#define BKG_FORMAT_OGG       0x02
#define BKG_FORMAT_MOD       0x03
#define BKG_FORMAT_USER      0xFF

#define BKG_PLAY_LOOP        0x100

enum getAudioInfoData {
	AI_CHANNELS,
	AI_FREQ,
	AI_BITS,
	AI_MAXVOICES,
	AI_PLAYINGVOICES,
	AI_MAXSAMPLES,
	AI_SAMPLES,
	AI_PAUSED,
	AI_BGSTREAM,
	AI_BUFFERSIZE,
	AI_NUMINFOITEMS
};


/* ctrlSample() ID Tags */

#define CTRL_S_END    0x00
#define CTRL_S_VOLUME 0x10
#define CTRL_S_PAN    0x11
#define CTRL_S_PLAYSTART 0x12
#define CTRL_S_PLAYEND   0x13
#define CTRL_S_LOOPSTART 0x14
#define CTRL_S_LOOPEND   0x15
#define CTRL_S_PRIORITY  0x16
#define CTRL_S_LOOPCOUNT 0x17
#define CTRL_S_ARRAY     0xFF

/* ctrlDevice() ID Tags */

#define CTRL_D_END	0x00
#define CTRL_D_VOLUME	0x10
#define CTRL_D_PAN	0x11
#define CTRL_D_ARRAY    0xFF

/* getSystemInfo bytes and defines */
enum getSystemInfoBytes {
   SYS_FREQ,
   SYS_BITS,
   SYS_CHAN,
   SYS_MISC,
   SYS_CPUA,
   SYS_CPUL,
   SYS_CPUR,
   SYS_GHZ,
   SYS_MHZ,
   SYS_OS,
   SYS_OSV1,
   SYS_OSV2,
   SYS_OSVB,
   SYS_ENDI,
   SYS_NUMVARS
   };
   
#define SYS_FREQ_ARBITRARY 1
#define SYS_FREQ_8000      2
#define SYS_FREQ_11025     4
#define SYS_FREQ_22050     8
#define SYS_FREQ_44100    16
#define SYS_FREQ_48000    32
#define SYS_FREQ_96000    64
   
#define SYS_BITS_8         1
#define SYS_BITS_16        2
#define SYS_BITS_24        4
#define SYS_BITS_32        8
#define SYS_BITS_FLOAT    16

#define SYS_MISC_DSP       1
#define SYS_MISC_ARAM      2


/* ERROR CODES */

#define ERR_NOERROR 0

#define ERR_NOT_INITIALIZED -32001
#define ERR_DEVICE_OPEN     -32002
#define ERR_CTRL_ALLOC      -32003
#define ERR_NOT_SUPPORTED   -32004 /* By 1.0 user should never get this error */
#define ERR_MEMORY_ALLOC    -32005

#define ERR_PLAYBUFFER_ALLOC -32007
#define ERR_SYSINFO         -32009

#define ERR_MUTEX_INIT      -32010
#define ERR_THREAD_START    -32011
#define ERR_THREAD_STOP     -32012

#define ERR_SAMPLES_FULL   -32020
#define ERR_SAMPLES_ALLOC  -32021
#define ERR_SAMPLES_FILE   -32022
#define ERR_SAMPLES_ATTRIB -32023
#define ERR_SAMPLES_RANGE  -32024

#define ERR_BKG_START -32030
#define ERR_BKG_FORMAT -32031
#define ERR_BKG_NOTPLAYING -32032

#define ERR_VOICES_ALLOC     -32040
#define ERR_VOICES_FULL      -32041

#define ERR_PLAYING_ID       -32045
#define ERR_PLAYING_LOOP     -32046

#define ERR_DEVICE_UNSUPPORTED  -32050
#define ERR_DEVICE_UNKNOWN      -32051
#define ERR_DEVICE_SUSPEND      -32055
#define ERR_DEVICE_RESUME       -32056

#define ERR_HARDWARE_PARMS	-32070
#define ERR_HARDWARE_BITS	-32071
#define ERR_HARDWARE_CHAN	-32072
#define ERR_HARDWARE_FREQ	-32073
#define ERR_HARDWARE_FRAG	-32074


/* Macros */

#define sssd_isError(a) (a < -32000 ? 1 : 0)

#define sssd_audioValid(ga)  (ga->audio_fd > -1)
#define sssd_audioPaused(ga) (ga->paused != 0)
#define audioValid(ga)  (ga->audio_fd > -1)
#define audioPaused(ga) (ga->paused != 0)

#define sssd_copySample(a,b,c,d,e,f,g) sssd_copySampleStereo(a,b,c,d,e,f,COPY_STEREO|g)
#define sssd_copySampleLeft(a,b,c,d,e,f,g) sssd_copySampleStereo(a,b,c,d,e,f,COPY_LEFT|g)
#define sssd_copySampleRight(a,b,c,d,e,f,g) sssd_copySampleStereo(a,b,c,d,e,f,COPY_RIGHT|g)

#define sssd_playSample(a,b,c,d,e) sssd_playSamplePP(a,b,0,c,d,e, SAMPLE_DEFAULTPOS, SAMPLE_DEFAULTPOS, SAMPLE_DEFAULTPOS, SAMPLE_DEFAULTPOS, PRIQ_NONE)
/* Prototypes */

#ifdef __cplusplus
extern "C"
{
#endif

#ifndef __ELATE__

#ifdef __SYMBIAN32__
int sssd_startAudio( struct sssdaudio *ga );
void outputNum(int num);
#else
_WINDOWS_DLL_PROTO int sssd_startAudio( struct sssdaudio* ga, const char* device_name );
#endif

_WINDOWS_DLL_PROTO int sssd_stopAudio( struct sssdaudio* ga );
_WINDOWS_DLL_PROTO void sssd_pauseAudio( struct sssdaudio* ga );
_WINDOWS_DLL_PROTO void sssd_unpauseAudio( struct sssdaudio* ga );

_WINDOWS_DLL_PROTO int sssd_copySampleStereo( struct sssdaudio* ga, void* data, int channels, int freq, int bits, int samples, int flags );
_WINDOWS_DLL_PROTO int sssd_registerSample( struct sssdaudio* ga,void *sampledata, int channels, int freq, int bits, int samples, int flags);

_WINDOWS_DLL_PROTO int sssd_playSamplePP( struct sssdaudio* ga, int sample_id, int flags, int repeat_count, int volume, int pan, int playStart, int playEnd, int loopStart, int loopEnd, int priorityQueue );
_WINDOWS_DLL_PROTO void sssd_stopSample( struct sssdaudio* ga, int sample_id );
_WINDOWS_DLL_PROTO void sssd_stopAllSamples( struct sssdaudio* ga );
_WINDOWS_DLL_PROTO int sssd_blockUntilComplete(struct sssdaudio *ga, int playing_id);
_WINDOWS_DLL_PROTO int sssd_blockUntilSample(struct sssdaudio *ga, int playing_id, int sample_num);
_WINDOWS_DLL_PROTO int sssd_timePlaying(struct sssdaudio *ga, int playing_id);

_WINDOWS_DLL_PROTO int sssd_ctrlSample( struct sssdaudio* ga, int sample_id, int *argArray);
_WINDOWS_DLL_PROTO int sssd_ctrlDevice( struct sssdaudio* ga, float *argArray);
_WINDOWS_DLL_PROTO int sssd_ctrlBackground( struct sssdaudio* ga, char *command);

_WINDOWS_DLL_PROTO int sssd_playBackground( struct sssdaudio* ga, const char* filename, int flags, char * user_args);
_WINDOWS_DLL_PROTO void sssd_stopBackground( struct sssdaudio* ga );
_WINDOWS_DLL_PROTO int sssd_checkBackground( struct sssdaudio* ga );
_WINDOWS_DLL_PROTO float sssd_timeBackground( struct sssdaudio* ga );

_WINDOWS_DLL_PROTO void sssd_freeSamples(struct sssdaudio *ga);

_WINDOWS_DLL_PROTO struct sssdaudio* sssd_initAudio(int channels, int freq, int bits, int max_voices, int max_samples, int buffersize, int numbuffers, int flags);
_WINDOWS_DLL_PROTO void sssd_freeAudio(struct sssdaudio *ga);

_WINDOWS_DLL_PROTO int sssd_getAudioInfo(struct sssdaudio *ga, int *stats, int numItems);
_WINDOWS_DLL_PROTO char *sssd_getSystemInfo(char *buffer, int sizeOfBuffer);
_WINDOWS_DLL_PROTO int sssd_testLatency(struct sssdaudio *ga);

_WINDOWS_DLL_PROTO int sssd_suspend(struct sssdaudio *ga);
_WINDOWS_DLL_PROTO int sssd_resume(struct sssdaudio *ga, const char* device_name); 

_WINDOWS_DLL_PROTO int sssd_resampleRaw(struct sssdaudio *ga, void *insamp, int infreq, int insize, int inchannels, int inbits, void *outdata, int outfreq, int flags);

_WINDOWS_DLL_PROTO void* sssd_getBuffer(struct sssdaudio *ga);
_WINDOWS_DLL_PROTO void sssd_sendBuffer(struct sssdaudio *ga, int bufnum);
_WINDOWS_DLL_PROTO int  sssd_isBufferPlaying(struct sssdaudio *ga, int bufnum);
_WINDOWS_DLL_PROTO int  sssd_isPlaying(struct sssdaudio *ga, int playing_id);

#else
  #include <amiga/sssd_elate.h>
#endif

#ifdef __cplusplus
}
#endif 


#endif
