#include "SSSDThreadHandler.h"
#include "SSSDThread.h"


#include <libc/sys/reent.h>


_LIT(KSSSDThreadName, "SSSDServerThread");


/* ========================================================================
 * Public API
 */

CSSSDThreadHandler *CSSSDThreadHandler::NewL() {
    CSSSDThreadHandler *self = new (ELeave) CSSSDThreadHandler();
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(self);
    return self;
}

void CSSSDThreadHandler::ConstructL() {
    User::LeaveIfError(iSSSDThread.Create(KSSSDThreadName,
                                          CSSSDThread::ThreadFunction,
                                          KDefaultStackSize,
                                          NULL,
                                          NULL));

    // give all possible priority to audio
    iSSSDThread.SetPriority(EPriorityRealTime);
    RThread().SetPriority(EPriorityAbsoluteLow);

    User::LeaveIfError(iShared.iAliveSem.CreateLocal(1));
    User::LeaveIfError(iShared.iStreamOpenedSem.CreateLocal(0));
    User::LeaveIfError(iShared.iMutex.CreateLocal());

	iPaused = ETrue;
}

CSSSDThreadHandler::CSSSDThreadHandler() {
}

CSSSDThreadHandler::~CSSSDThreadHandler() {
    if (iPaused) {
        SendCmd(ECmdStartMixer);
    }

    SendCmd(ECmdDestroyMixer);

	iShared.iStreamOpenedSem.Close();
    iShared.iAliveSem.Wait();
    iShared.iAliveSem.Close();
    iShared.iMutex.Close();

    CloseSTDLIB();
}

void CSSSDThreadHandler::SSSD_StartAudio(struct sssdaudio* ga ) {
    iShared.iSSSDAudio = ga;
    iSSSDThread.SetInitialParameter(&iShared);
    iSSSDThread.Resume();

    Resume();
}

void CSSSDThreadHandler::SSSD_StopAudio() {
    Pause();
}

void CSSSDThreadHandler::SendCmd(TMixerCmd aCmd) {
	iShared.iMutex.Wait();
	iShared.iCmd = aCmd;
	iShared.iMutex.Signal();
	iSSSDThread.RaiseException(EExcUserInterrupt);
}

void CSSSDThreadHandler::Pause() {
    if (iPaused) return;
    iPaused = ETrue;
    SendCmd(ECmdStopMixer);
}

void CSSSDThreadHandler::Resume() {
    if (!iPaused) return;

    iPaused = EFalse;
	SendCmd(ECmdStartMixer);
}

