#ifndef __SSSD_THREADHANDLER_H__
#define __SSSD_THREADHANDLER_H__

#include <e32base.h>
#include "TAudioShared.h"

class CSSSDAudioOutputStream;


/*!
   @class CSSSDThreadHandler
   
   @discussion
        Handler that allows SSSD to run in its own Symbian background thread
 */
class CSSSDThreadHandler : public CBase {


    /* ========================================================================
     * Private instance data
     */

    TAudioShared iShared;   // shared data with sssd thread
    RThread iSSSDThread;    // handle to sssd thread
    TBool iPaused;
    
     
    /* ========================================================================
     * Private methods
     */
    
     
    /*!
       @function CSSSDThreadHandler
       
       @discussion
            Construct an instance of the thread handler (non-leaving code)
     */
    CSSSDThreadHandler();
    
    
    /*!
       @function ConstructL
       
       @discussion
            Any construction that can leave is in this method
     */
    void ConstructL();
    
    void SendCmd(TMixerCmd aCmd);
    
    
    /* ========================================================================
     * Public API
     */
     
public:

    static CSSSDThreadHandler* NewL();
    
    /*!
       @function ~CSSSDThreadHandler
       
       @discussion
            Object destructor - cleans up all resources held by the object.
            Automatically called when the object is being deleted.
     */
    ~CSSSDThreadHandler();
    
    /*!
       @function SSSD_StartAudio
       
       @discussion
            Sets up the SSSD audio output stream and launches it in a new thread,
            so that the SSSD mixer runs asynchronously.
     */
    void SSSD_StartAudio(struct sssdaudio* ga);
    void SSSD_StopAudio();
    void Pause();
    void Resume();
    
};

#endif /* __SSSD_THREADHANDLER_H__ */
