/* Copyright (c) 2002, Nokia. All rights reserved */

// REMARK: added this...
#include <bautils.h>
#include <eikappui.h>
#include <eikapp.h>

#include <coemain.h>
#include <HelloWorld.rsg>

#include "HelloWorldAppView.h"
#include <stdlib.h>

extern "C" {
    #include "loadSimpleWave.h"
}


// Standard construction sequence
CHelloWorldAppView* CHelloWorldAppView::NewL(const TRect& aRect) {
    CHelloWorldAppView* self = CHelloWorldAppView::NewLC(aRect);
    CleanupStack::Pop(self);
    return self;
}

CHelloWorldAppView* CHelloWorldAppView::NewLC(const TRect& aRect) {
    CHelloWorldAppView* self = new (ELeave) CHelloWorldAppView;
    CleanupStack::PushL(self);
    self->ConstructL(aRect);
    return self;
}

CHelloWorldAppView::CHelloWorldAppView() {
	// no implementation required
}


CHelloWorldAppView::~CHelloWorldAppView() {

    if(ga!=NULL) {
        sssd_stopAudio(ga);
	    sssd_freeAudio(ga);
	    ga = NULL;
    }

    CloseSTDLIB();
}

void CHelloWorldAppView::PlaySound() {
    /* play sample */
    hello_play=sssd_playSample( ga, hello_id, 1, 256, 0 );
    
    /* Wait until the sample has finished playing */
    sssd_blockUntilComplete(ga, hello_play);
}


void CHelloWorldAppView::ConstructL(const TRect& aRect) {
    // Create a window for this application view
    CreateWindowL();

    // Set the windows size
    SetRect(aRect);

    // Activate the window, which makes it ready to be drawn
    ActivateL();

    /* wave file info */
    waveFile *waveData;

    // initialise the SSSD server thread
    ga=sssd_initAudio(1,16000,8,4,24,0,0,0);
    User::LeaveIfNull(ga);
    if(sssd_isError(sssd_startAudio(ga)) ) {
        free(ga);
        ga = NULL;
        User::LeaveNoMemory();
    }
    
    /* load simple Wav file */
    waveData=loadSimpleWave("C:\\system\\apps\\HelloWorld\\hello_16000.wav");
    User::LeaveIfNull(waveData);

    /* pre-cache samples ... keeping id's */
    /* simpleWave2copySample is a macro that fills in args to sssd_copySample */
    /* when used with loadSimpleWave() */
    hello_id   = simpleWave2copySample( ga, waveData, COPY_DONT_RESAMPLE);
    if (sssd_isError(hello_id)) {
        User::LeaveIfError(hello_id);
    }
    
    /* As soon as sample is copy cached, we can delete the local copy */
    free(waveData);
    
    

}

// Draw this application's view to the screen
void CHelloWorldAppView::Draw(const TRect& /*aRect*/) const {
    // Get the standard graphics context
    CWindowGc& gc = SystemGc();

    // Gets the control's extent
    TRect rect = Rect();

    // Clears the screen
    gc.Clear(rect);
}

void CHelloWorldAppView::Pause() {
    sssd_pauseAudio(ga);
}

void CHelloWorldAppView::Unpause() {
    sssd_unpauseAudio(ga);
}
